==> tuyaos_demo_quickstart_0.0.6.tar.gz: tuyaos_demo_quickstart/src/tuya_device.c <==
/**
 * @file tuya_device.c
 * @brief tuyaos quick start demo
 * @version 1.0
 * @date 2022-05-09
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */

#define _TUYA_DEVICE_GLOBAL

/***********************************************************************
 ** INCLUDE                                                           **
 **********************************************************************/
#include "tuya_cloud_types.h"
#include "tuya_cloud_com_defs.h"
#include "tuya_iot_wifi_api.h"
#include "gw_intf.h"
#include "uni_log.h"
#include "dp_process.h"
#include "app_key.h"
#include "app_led.h"
#if defined(ENABLE_LWIP) && (ENABLE_LWIP == 1)
#include "lwip_init.h"
#endif
#if defined(ENABLE_PRODUCT_AUTOTEST) && (ENABLE_PRODUCT_AUTOTEST == 1)
#include "prod_test.h"
#endif
/***********************************************************************
 ** CONSTANT ( MACRO AND ENUM )                                       **
 **********************************************************************/

/***********************************************************************
 ** STRUCT                                                            **
 **********************************************************************/

/***********************************************************************
 ** VARIABLE                                                          **
 **********************************************************************/

/***********************************************************************
 ** FUNCTON                                                           **
 **********************************************************************/

// User TODO: product information
// Note that if you use the default authorization information of the code, there may be problems of multiple users and conflicts, 
// so try to use all the authorizations purchased from the tuya iot platform,
// Buying guide：https://developer.tuya.com/en/docs/iot/lisence-management?id=Kb4qlem97idl0
// You can also apply for two authorization codes for free in the five-step hardware development stage of the Tuya IoT platform.
// The authorization information can also be written through the production test tool. 
// When the production test function is started and the authorization is burned with the Tuya cloud module tool, 
// please comment out this piece of code.

#define PID "uiiyltkjmkhwumga"
#define UUID "tuya60dcbf014a5da4c6"
#define AUTHKEY "GPwI9eWvUFzYP3iGMnpx2wa18jq61maj"

// User TODO: Some information inside the firmware is defined here, developers can change the firmware information by modifying here
#ifndef USER_SW_VER
#define USER_SW_VER "1.0.0"
#endif

/**
* @brief  Upgrade file download complete callback
*
* @param[in] fw: firmware info
* @param[in] download_result: 0 means download succes. other means fail
* @param[in] pri_data: private data
* @return none
*/
STATIC OPERATE_RET __ota_upgrade_notify_cb(IN CONST FW_UG_S *fw, IN CONST INT_T download_result, IN PVOID_T pri_data)
{
    if (download_result == 0) {
        PR_DEBUG("SOC Upgrade File Download Success");
        // UserTODO
    } else {
        PR_ERR("SOC Upgrade File Download Fail.ret = %d", download_result);
    }

    return OPRT_OK;
}

/**
* @brief firmware download content process callback
*
* @param[in] fw: firmware info
* @param[in] total_len: firmware total size
* @param[in] offset: offset of this download package
* @param[in] data && len: this download package
* @param[out] remain_len: the size left to process in next cb
* @param[in] pri_data: private data
* @return OPRT_OK: success  Other: fail
*/
STATIC OPERATE_RET __ota_get_file_data_cb(IN CONST FW_UG_S *fw, IN CONST UINT_T total_len, IN CONST UINT_T offset,
                                          IN CONST BYTE_T *data, IN CONST UINT_T len, OUT UINT_T *remain_len, IN PVOID_T pri_data)
{
    PR_DEBUG("Rev File Data");
    PR_DEBUG("Total_len:%u", total_len);
    PR_DEBUG("Offset:%u Len:%u", offset, len);

    // UserTODO
    return OPRT_OK;
}

/**
* @brief SOC device upgrade entry
*
* @param[in] fw: firmware info
* @return 
*/
STATIC INT_T __soc_dev_rev_upgrade_info_cb(IN CONST FW_UG_S *fw)
{
    PR_DEBUG("SOC Rev Upgrade Info");
    PR_DEBUG("fw->tp:%d", fw->tp);
    PR_DEBUG("fw->fw_url:%s", fw->fw_url);
    PR_DEBUG("fw->fw_hmac:%s", fw->fw_hmac);
    PR_DEBUG("fw->sw_ver:%s", fw->sw_ver);
    PR_DEBUG("fw->file_size:%u", fw->file_size);

    OPERATE_RET op_ret = tuya_iot_upgrade_gw(fw, __ota_get_file_data_cb, __ota_upgrade_notify_cb, NULL);
    if(OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_upgrade_gw err:%d",op_ret);
    }

    // UserTODO
    return OPRT_OK;
}

/**
* @brief SOC device cloud state change callback
*
* @param[in] status: current status
* @return none
*/
STATIC VOID_T __soc_dev_status_changed_cb(IN CONST GW_STATUS_E status)
{
    PR_DEBUG("SOC TUYA-Cloud Status:%d", status);
}

// SOC device specific data query entry
STATIC VOID_T __soc_dev_dp_query_cb(IN CONST TY_DP_QUERY_S *dp_qry)
{
    PR_DEBUG("SOC Rev DP Query Cmd");
    if (dp_qry->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp_qry->cid);
    }

    if (dp_qry->cnt == 0) {
        PR_DEBUG("soc rev all dp query");
        update_all_dp();
    } else {
        PR_DEBUG("soc rev dp query cnt:%d", dp_qry->cnt);
        UINT_T index = 0;
        for (index = 0; index < dp_qry->cnt; index++) {
            PR_DEBUG("rev dp query:%d", dp_qry->dpid[index]);
            // UserTODO
        }
    }
}

/**
* @brief  SOC device format command data delivery entry
*
* @param[in] dp: obj dp info
* @return none
*/
STATIC VOID_T __soc_dev_obj_dp_cmd_cb(IN CONST TY_RECV_OBJ_DP_S *dp)
{
    PR_DEBUG("SOC Rev DP Obj Cmd t1:%d t2:%d CNT:%u", dp->cmd_tp, dp->dtt_tp, dp->dps_cnt);
    if (dp->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp->cid);
    }
    UINT_T index = 0;
    for (index = 0; index < dp->dps_cnt; index++) {
        deal_dp_proc(&(dp->dps[index]));
    }

    return;
}

/**
* @brief SOC device transparently transmits command data delivery entry
*
* @param[in] dp: raw dp info
* @return none
*/
STATIC VOID_T __soc_dev_raw_dp_cmd_cb(IN CONST TY_RECV_RAW_DP_S *dp)
{
    PR_DEBUG("SOC Rev DP Raw Cmd t1:%d t2:%d dpid:%d len:%u", dp->cmd_tp, dp->dtt_tp, dp->dpid, dp->len);
    if (dp->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp->cid);
    }
}

/**
* @brief SOC device process restart request entry
*
* @param[in] type: gateway reset type
* @return none
*/
STATIC VOID_T __soc_dev_restart_req_cb(GW_RESET_TYPE_E type)
{
    PR_DEBUG("SOC Rev Restart Req %d", type);
    if (GW_RESET_DATA_FACTORY != type) {
        // UserTODO Device process restart
        tal_system_reset();
    }
}

/**
* @brief  SOC external network status change callback
*
* @param[in] stat: curr network status
* @return none
*/
STATIC VOID __soc_dev_net_status_cb(IN CONST GW_WIFI_NW_STAT_E stat)
{
    PR_DEBUG("network status:%d", stat);
}

/**
* @brief  SOC device initialization
*
* @param[in] none
* @return OPRT_OK:success,other:fail.
*/
OPERATE_RET __soc_device_init(VOID_T)
{
    OPERATE_RET op_ret = OPRT_OK;

    // Set authorization information
    // Note that if you use the default authorization information of the code, there may be problems of multiple users and conflicts, 
    // so try to use all the authorizations purchased from the tuya iot platform,    
    // Buying guide://developer.tuya.com/en/docs/iot/lisence-management?id=Kb4qlem97idl0
    // You can also apply for two authorization codes for free in the five-step hardware development stage of the Tuya IoT platform.
    // Authorization information can also be written through the production testing tool. 
    // When the production testing function is started and the authorization is burned with the Tuya Cloud module tool, 
    // please comment out this piece of code.    
    WF_GW_PROD_INFO_S prod_info = {UUID, AUTHKEY};
    op_ret = tuya_iot_set_wf_gw_prod_info(&prod_info);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_set_gw_prod_info err:%d", op_ret);
        return -2;
    }

    // Initialize TuyaOS product information
    TY_IOT_CBS_S iot_cbs = {0};
    iot_cbs.gw_status_cb = __soc_dev_status_changed_cb;
    iot_cbs.gw_ug_cb = __soc_dev_rev_upgrade_info_cb;
    iot_cbs.gw_reset_cb = __soc_dev_restart_req_cb;
    iot_cbs.dev_obj_dp_cb = __soc_dev_obj_dp_cmd_cb;
    iot_cbs.dev_raw_dp_cb = __soc_dev_raw_dp_cmd_cb;
    iot_cbs.dev_dp_query_cb = __soc_dev_dp_query_cb;

    op_ret = tuya_iot_wf_soc_dev_init(GWCM_OLD, WF_START_SMART_AP_CONCURRENT, &iot_cbs, PID, USER_SW_VER);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_wf_soc_dev_init err:%d", op_ret);
        return -1;
    }

    op_ret = tuya_iot_reg_get_wf_nw_stat_cb(__soc_dev_net_status_cb);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_reg_get_wf_nw_stat_cb err:%d", op_ret);
        return op_ret;
    }

    app_led_init();
    app_key_init();

    return 0;
}

/**
* @brief Initialization device
*
* @param[in] none
* @return none
*/
STATIC VOID_T user_main(VOID_T)
{
    OPERATE_RET rt = OPRT_OK;

    // Initialization, because DB initialization takes a long time, 
    // which affects the startup efficiency of some devices, 
    // so special processing is performed during initialization to delay initialization of DB    
    TY_INIT_PARAMS_S init_param = {0};
    init_param.init_db = TRUE;
    strcpy(init_param.sys_env, TARGET_PLATFORM);
    TUYA_CALL_ERR_LOG(tuya_iot_init_params(NULL, &init_param));

    // Initialization LWIP
#if defined(ENABLE_LWIP) && (ENABLE_LWIP == 1)
    TUYA_LwIP_Init();
#endif

    // Support the production test of finished products, 
    // and test and verify the equipment in the production stage through wireless means
#if defined(ENABLE_PRODUCT_AUTOTEST) && (ENABLE_PRODUCT_AUTOTEST == 1)
    if (prodtest_ssid_scan(500)) {
        return;
    }

    // Initialization device
    PR_DEBUG("device_init in");
    TUYA_CALL_ERR_LOG(__soc_device_init());
#else
    // The function is activated, in which the GWCM_OLD mode is special, 
    // and the SSID can be scanned at any time to enter the production test mode.
    GW_WF_CFG_MTHD_SEL gwcm_mode = GWCM_OLD;
    PR_DEBUG("gwcm_mode %d", gwcm_mode); //No low power mode by default
    if (gwcm_mode != GWCM_OLD) {
        PR_DEBUG("low_power select");
        // Scan to the production test route, 
        // enter the production test mode, no need to initialize the device
        #if 0
        if (TRUE == scan_test_ssid()) {
            PR_DEBUG("prodtest");
            return;
        }
        #endif

        // The production test route is not scanned, and the device is initialized
        PR_DEBUG("no tuya_mdev_test1!");
        PR_DEBUG("device_init in");
        TUYA_CALL_ERR_LOG(__soc_device_init());
    } else {
        // Initialize device
        PR_DEBUG("device_init in");
        TUYA_CALL_ERR_LOG(__soc_device_init());
    }
#endif
    return;
}

//Create a task handle
THREAD_HANDLE ty_app_thread = NULL;

/**
* @brief  task thread
*
* @param[in] arg:Parameters when creating a task
* @return none
*/
STATIC VOID_T tuya_app_thread(VOID_T *arg)
{
    user_main();

    tal_thread_delete(ty_app_thread);
    ty_app_thread = NULL;
}

/**
* @brief   user entry function
*
* @param[in] none
* @return none
*/
VOID_T tuya_app_main(VOID_T)
{
    THREAD_CFG_T thrd_param = {4096, 4, "tuya_app_main"};
    tal_thread_create_and_start(&ty_app_thread, NULL, NULL, tuya_app_thread, NULL, &thrd_param);
}

==> tuyaos_demo_quickstart_0.0.7.tar.gz: tuyaos_demo_quickstart/src/tuya_device.c <==
/**
 * @file tuya_device.c
 * @brief tuyaos quick start demo
 * @version 1.0
 * @date 2022-05-09
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */

#define _TUYA_DEVICE_GLOBAL

/***********************************************************************
 ** INCLUDE                                                           **
 **********************************************************************/
#include "tuya_cloud_types.h"
#include "tuya_cloud_com_defs.h"
#include "tuya_iot_wifi_api.h"
#include "gw_intf.h"
#include "uni_log.h"
#include "dp_process.h"
#include "app_key.h"
#include "app_led.h"
#if defined(ENABLE_LWIP) && (ENABLE_LWIP == 1)
#include "lwip_init.h"
#endif

/***********************************************************************
 ** CONSTANT ( MACRO AND ENUM )                                       **
 **********************************************************************/

/***********************************************************************
 ** STRUCT                                                            **
 **********************************************************************/

/***********************************************************************
 ** VARIABLE                                                          **
 **********************************************************************/

/***********************************************************************
 ** FUNCTON                                                           **
 **********************************************************************/

// User TODO: product information
// Note that if you use the default authorization information of the code, there may be problems of multiple users and conflicts, 
// so try to use all the authorizations purchased from the tuya iot platform,
// Buying guide：https://developer.tuya.com/en/docs/iot/lisence-management?id=Kb4qlem97idl0
// You can also apply for two authorization codes for free in the five-step hardware development stage of the Tuya IoT platform.
// The authorization information can also be written through the production test tool. 
// When the production test function is started and the authorization is burned with the Tuya cloud module tool, 
// please comment out this piece of code.

#define PID "uiiyltkjmkhwumga"
#define UUID "tuya60dcbf014a5da4c6"
#define AUTHKEY "GPwI9eWvUFzYP3iGMnpx2wa18jq61maj"

// User TODO: Some information inside the firmware is defined here, developers can change the firmware information by modifying here
#ifndef USER_SW_VER
#define USER_SW_VER "1.0.0"
#endif

/**
* @brief  Upgrade file download complete callback
*
* @param[in] fw: firmware info
* @param[in] download_result: 0 means download succes. other means fail
* @param[in] pri_data: private data
* @return none
*/
STATIC OPERATE_RET __ota_upgrade_notify_cb(IN CONST FW_UG_S *fw, IN CONST INT_T download_result, IN PVOID_T pri_data)
{
    if (download_result == 0) {
        PR_DEBUG("SOC Upgrade File Download Success");
        // UserTODO
    } else {
        PR_ERR("SOC Upgrade File Download Fail.ret = %d", download_result);
    }

    return OPRT_OK;
}

/**
* @brief firmware download content process callback
*
* @param[in] fw: firmware info
* @param[in] total_len: firmware total size
* @param[in] offset: offset of this download package
* @param[in] data && len: this download package
* @param[out] remain_len: the size left to process in next cb
* @param[in] pri_data: private data
* @return OPRT_OK: success  Other: fail
*/
STATIC OPERATE_RET __ota_get_file_data_cb(IN CONST FW_UG_S *fw, IN CONST UINT_T total_len, IN CONST UINT_T offset,
                                          IN CONST BYTE_T *data, IN CONST UINT_T len, OUT UINT_T *remain_len, IN PVOID_T pri_data)
{
    PR_DEBUG("Rev File Data");
    PR_DEBUG("Total_len:%u", total_len);
    PR_DEBUG("Offset:%u Len:%u", offset, len);

    // UserTODO
    return OPRT_OK;
}

/**
* @brief SOC device upgrade entry
*
* @param[in] fw: firmware info
* @return 
*/
STATIC INT_T __soc_dev_rev_upgrade_info_cb(IN CONST FW_UG_S *fw)
{
    PR_DEBUG("SOC Rev Upgrade Info");
    PR_DEBUG("fw->tp:%d", fw->tp);
    PR_DEBUG("fw->fw_url:%s", fw->fw_url);
    PR_DEBUG("fw->fw_hmac:%s", fw->fw_hmac);
    PR_DEBUG("fw->sw_ver:%s", fw->sw_ver);
    PR_DEBUG("fw->file_size:%u", fw->file_size);

    OPERATE_RET op_ret = tuya_iot_upgrade_gw(fw, __ota_get_file_data_cb, __ota_upgrade_notify_cb, NULL);
    if(OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_upgrade_gw err:%d",op_ret);
    }

    // UserTODO
    return OPRT_OK;
}

/**
* @brief SOC device cloud state change callback
*
* @param[in] status: current status
* @return none
*/
STATIC VOID_T __soc_dev_status_changed_cb(IN CONST GW_STATUS_E status)
{
    PR_DEBUG("SOC TUYA-Cloud Status:%d", status);
}

// SOC device specific data query entry
STATIC VOID_T __soc_dev_dp_query_cb(IN CONST TY_DP_QUERY_S *dp_qry)
{
    PR_DEBUG("SOC Rev DP Query Cmd");
    if (dp_qry->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp_qry->cid);
    }

    if (dp_qry->cnt == 0) {
        PR_DEBUG("soc rev all dp query");
        update_all_dp();
    } else {
        PR_DEBUG("soc rev dp query cnt:%d", dp_qry->cnt);
        UINT_T index = 0;
        for (index = 0; index < dp_qry->cnt; index++) {
            PR_DEBUG("rev dp query:%d", dp_qry->dpid[index]);
            // UserTODO
        }
    }
}

/**
* @brief  SOC device format command data delivery entry
*
* @param[in] dp: obj dp info
* @return none
*/
STATIC VOID_T __soc_dev_obj_dp_cmd_cb(IN CONST TY_RECV_OBJ_DP_S *dp)
{
    PR_DEBUG("SOC Rev DP Obj Cmd t1:%d t2:%d CNT:%u", dp->cmd_tp, dp->dtt_tp, dp->dps_cnt);
    if (dp->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp->cid);
    }
    UINT_T index = 0;
    for (index = 0; index < dp->dps_cnt; index++) {
        deal_dp_proc(&(dp->dps[index]));
    }

    return;
}

/**
* @brief SOC device transparently transmits command data delivery entry
*
* @param[in] dp: raw dp info
* @return none
*/
STATIC VOID_T __soc_dev_raw_dp_cmd_cb(IN CONST TY_RECV_RAW_DP_S *dp)
{
    PR_DEBUG("SOC Rev DP Raw Cmd t1:%d t2:%d dpid:%d len:%u", dp->cmd_tp, dp->dtt_tp, dp->dpid, dp->len);
    if (dp->cid != NULL) {
        PR_ERR("soc not have cid.%s", dp->cid);
    }
}

/**
* @brief SOC device process restart request entry
*
* @param[in] type: gateway reset type
* @return none
*/
STATIC VOID_T __soc_dev_restart_req_cb(GW_RESET_TYPE_E type)
{
    PR_DEBUG("SOC Rev Restart Req %d", type);
    if (GW_RESET_DATA_FACTORY != type) {
        // UserTODO Device process restart
        tal_system_reset();
    }
}

/**
* @brief  SOC external network status change callback
*
* @param[in] stat: curr network status
* @return none
*/
STATIC VOID __soc_dev_net_status_cb(IN CONST GW_WIFI_NW_STAT_E stat)
{
    PR_DEBUG("network status:%d", stat);
}

/**
* @brief  SOC device initialization
*
* @param[in] none
* @return OPRT_OK:success,other:fail.
*/
OPERATE_RET __soc_device_init(VOID_T)
{
    OPERATE_RET op_ret = OPRT_OK;

    // Set authorization information
    // Note that if you use the default authorization information of the code, there may be problems of multiple users and conflicts, 
    // so try to use all the authorizations purchased from the tuya iot platform,    
    // Buying guide://developer.tuya.com/en/docs/iot/lisence-management?id=Kb4qlem97idl0
    // You can also apply for two authorization codes for free in the five-step hardware development stage of the Tuya IoT platform.
    // Authorization information can also be written through the production testing tool. 
    // When the production testing function is started and the authorization is burned with the Tuya Cloud module tool, 
    // please comment out this piece of code.    

    //(1) 为了方便开发者快速体验和调试，此处直接调用tuya_iot_set_wf_gw_prod_info接口。
    //(2) 在实际生产过程中，如果采用涂鸦生产体系（产测上位机工具、测架），请参考mf_test example。

    //(1) In order to facilitate developers' quick experience and debugging, 
    //the tuya_iot_set_wf_gw_prod_info interface is called directly here.
    //(2)In the actual production process, if the Tuya production system (production and test host computer tools, test racks) is used, 
    //please refer to mf_test example.

    WF_GW_PROD_INFO_S prod_info = {UUID, AUTHKEY};
    op_ret = tuya_iot_set_wf_gw_prod_info(&prod_info);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_set_gw_prod_info err:%d", op_ret);
        return -2;
    }

    // Initialize TuyaOS product information
    TY_IOT_CBS_S iot_cbs = {0};
    iot_cbs.gw_status_cb = __soc_dev_status_changed_cb;
    iot_cbs.gw_ug_cb = __soc_dev_rev_upgrade_info_cb;
    iot_cbs.gw_reset_cb = __soc_dev_restart_req_cb;
    iot_cbs.dev_obj_dp_cb = __soc_dev_obj_dp_cmd_cb;
    iot_cbs.dev_raw_dp_cb = __soc_dev_raw_dp_cmd_cb;
    iot_cbs.dev_dp_query_cb = __soc_dev_dp_query_cb;

    op_ret = tuya_iot_wf_soc_dev_init(GWCM_OLD, WF_START_SMART_AP_CONCURRENT, &iot_cbs, PID, USER_SW_VER);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_wf_soc_dev_init err:%d", op_ret);
        return -1;
    }

    op_ret = tuya_iot_reg_get_wf_nw_stat_cb(__soc_dev_net_status_cb);
    if (OPRT_OK != op_ret) {
        PR_ERR("tuya_iot_reg_get_wf_nw_stat_cb err:%d", op_ret);
        return op_ret;
    }

    app_led_init();
    app_key_init();

    return 0;
}

/**
* @brief Initialization device
*
* @param[in] none
* @return none
*/
STATIC VOID_T user_main(VOID_T)
{
    OPERATE_RET rt = OPRT_OK;

    // Initialization, because DB initialization takes a long time, 
    // which affects the startup efficiency of some devices, 
    // so special processing is performed during initialization to delay initialization of DB    
    TY_INIT_PARAMS_S init_param = {0};
    init_param.init_db = TRUE;
    strcpy(init_param.sys_env, TARGET_PLATFORM);
    TUYA_CALL_ERR_LOG(tuya_iot_init_params(NULL, &init_param));

    // Initialization LWIP
#if defined(ENABLE_LWIP) && (ENABLE_LWIP == 1)
    TUYA_LwIP_Init();
#endif

    // Initialize device
    PR_DEBUG("device_init ");
    TUYA_CALL_ERR_LOG(__soc_device_init());

    return;
}

//Create a task handle
THREAD_HANDLE ty_app_thread = NULL;

/**
* @brief  task thread
*
* @param[in] arg:Parameters when creating a task
* @return none
*/
STATIC VOID_T tuya_app_thread(VOID_T *arg)
{
    user_main();

    tal_thread_delete(ty_app_thread);
    ty_app_thread = NULL;
}

/**
* @brief   user entry function
*
* @param[in] none
* @return none
*/
VOID_T tuya_app_main(VOID_T)
{
    THREAD_CFG_T thrd_param = {4096, 4, "tuya_app_main"};
    tal_thread_create_and_start(&ty_app_thread, NULL, NULL, tuya_app_thread, NULL, &thrd_param);
}
